%% reference: Xuchun Li, Lei Wang, Eric Sung - "A Study of AdaBoost with SVM Based Weak Learners" - IJCNN 2005
%% Para SVM la sigma del articulo es el learning rate (learn_rate_min = 0.001; learn_rate_max = 0.8; learn_rate_step = 0.05)
%% Para LVQ la sigma del articulo es el numero de hidden_neurons (hidden_neurons_min = 3; hidden_neurons_max = 20; hidden_neurons_step=1)

function [Accuracy Specificity Sensitivity alfa weakres] = Diverse_AdaBoost_LVQ_MeanAndStdDev (learning_function)
   

   load /home/maite/Documentos/IWANN_IWINAC_IDEAL2009/codigo_IWANN09/spm_segmented_oasis_females.mat

   %Intensidades de voxel
   %[C P] = extractVoxelIntensitiesWithinClusters (female_crystal_brain_cov0, CONTROLES, PACIENTES);
   
   %Para medias y stddev
   [C P] = extractMeanAndStdDevFromEachCluster   (female_crystal_brain_cov0, CONTROLES, PACIENTES);

   [L, num_clusters] = bwlabeln (female_crystal_brain_cov0);

   k_de_kfold = 10;
   p = cvpartition(size(P,1),'kfold',k_de_kfold);
   c = cvpartition(size(C,1),'kfold',k_de_kfold);
   
   control_label=2;
   patient_label=1;

   hidden_neurons_min  = 5; 
   hidden_neurons_max  = 20; 
   hidden_neurons_step = 1;
   
   min_diversity   = 0.5;
   
   
 %%Parametros por defecto de las LVQ
   epochs         = 150;
   goal           = 0.01;
   show           = Inf;
   showWindow     = false;
   learning_rate  = 0.01; %0.1

  
    
   
   for k=1:k_de_kfold;
       
    %classify and evaluate adaboost
       total_tests = 0;
       TP = 0;
       TN = 0;
       FP = 0;
       FN = 0;
       
      hidden_neurons    = hidden_neurons_max;
      n                = 1;
      total_diversity  = 0;
   
      %compute folds
      %training set used to compute the weights for each clusterSVM (our weaklearners)
      C_training = C(training (c,k) == 1,:,:);
      P_training = P(training (p,k) == 1,:,:);
   
      %test set used to validate the adaboost
      C_test     = C(training (c,k) == 0,:,:);
      P_test     = P(training (p,k) == 0,:,:);

      training_patients_no = size(P_training,1);
      training_controls_no = size(C_training,1);
      training_subjects_no = training_patients_no + training_controls_no;
      sample_weights       = ones(1,training_subjects_no)'/training_subjects_no; %sample_weights [#training_subjects x n]

      %initialize training set
      [training_data training_label feat_no] = alignAndLabelDataForNN (C_training, P_training, control_label, patient_label);
     
      label_ind = ind2vec(training_label);

      numTrain = training_subjects_no;
      priori_1 = training_controls_no/numTrain;
      priori_2 = training_patients_no/numTrain;

      %weight training
      while (hidden_neurons > hidden_neurons_min);
          hidden_neurons
          
          %creating neural network and training with training set
          lvqnet = newlvq(minmax(training_data'), hidden_neurons, [priori_1 priori_2], learning_rate, learning_function);
          lvqnet.trainParam.epochs     = epochs;
          lvqnet.trainParam.goal       = goal;
          lvqnet.trainParam.show       = show;
          lvqnet.trainParam.showWindow = showWindow;

          [trained_net, train_record] = train(lvqnet, training_data', label_ind);

         %classification test with training data
         predict_label = vec2ind ( sim (trained_net, training_data') ); 
         
          %Para poder comparar predict_label con subj_predictions_training
          %(ht(xi) con f(xi))
          trans_predict_label(predict_label==control_label) = 1;
          trans_predict_label(predict_label==patient_label) = -1;
          
          size(trans_predict_label)
         
        if (n>1);       
            
            aux = 0;
            for s=1:size(training_data,1); 
                 %adaboost compute training errors ¿ht(xi)=yi?
                if (predict_label(s) ~= training_label(s));
                    aux= aux + sample_weights(s,n);       
                end;
                %%Calculo Diversity ¿ht(xi)= f(xi)?
                if (trans_predict_label(s) ~= subj_predictions_training(s));
                    diversity      = diversity       + 1;
                end
            end;
            training_error = aux;
            size(training_data,1)
            diversity = diversity/size(training_data,1);
            
        else %% n = 1; la primera vez 'subj_predictions_training' no existe
            training_error = 0;
            diversity      = min_diversity;
        end
        
         training_error
         diversity
         
         %current_diversity = total_diversity + ( diversity/(n*training_subjects_no) );
         if ( (training_error > 0.50) || ( diversity < min_diversity ) )
            hidden_neurons = hidden_neurons - hidden_neurons_step
         else
            %save the weaklearner, save the world!
            weaklearners(n).net = trained_net;

            training_errors (n) = training_error;
            diversities     (n) = diversity;

            %compute total_diversity
            %total_diversity = current_diversity;

            %adaboost set classifier weights
            alfa(n) = 1;
            if (training_errors(n) ~= 0);
               alfa (n) = 0.5 * log((1-training_errors(n)) / training_errors(n));
            end;
            
            sample_weights (:,n+1) = sample_weights (:,n) .* exp( -alfa(n) .* training_label .* predict_label');
%             sample_weights (:,n+1) = sample_weights (:,n+1)/sum(sample_weights (:,n));
            sample_weights (:,n+1) = sample_weights (:,n+1)/100;
            
            
            %%inicializamos/actualizamos 'subj_predictions_training', es decir, f(x)
            subj_predictions_training = zeros (1, training_subjects_no);
            for i=1:size(alfa,1);
                subj_predictions_training = subj_predictions_training + (alfa(i).*trans_predict_label);
                subj_predictions_training = sign(subj_predictions_training)
            end;
            
            n = n + 1    
         end;
      end;

      %classify test set
      [test_data, test_label, feat_no] = alignAndLabelDataForNN (C_test, P_test, control_label, patient_label);
      label_ind=ind2vec(test_label);

      for i=1:size(alfa,1);
         boosted_results(k,i).output = vec2ind ( sim (weaklearners(i).net, test_data') );
         boosted_results(k,i).target = test_label;
         [weakres(k,i).TP weakres(k,i).TN weakres(k,i).FP weakres(k,i).FN weakres(k,i).Accuracy weakres(k,i).Specificity weakres(k,i).Sensitivity] = computeErrors (boosted_results(k,i).target', boosted_results(k,i).output);
      end;
      

    
      %Transformamos los labels controles=1 y pacientes=-1 para
      %calcular la clasificacion el cjto de weaklearners
       trans_boosted_results=boosted_results;
     for j=1:size(alfa,1);

          trans_boosted_results(k,j).output(boosted_results(k,j).output==control_label) = 1;
          trans_boosted_results(k,j).output(boosted_results(k,j).output==patient_label) =-1;

          trans_boosted_results(k,j).target(boosted_results(k,j).target==control_label) = 1;
          trans_boosted_results(k,j).target(boosted_results(k,j).target==patient_label) =-1;
      
     end
     
     trans_boosted_results(k,:).target
     
     for s=1:size(test_data,1);
         
         %calculamos f(x)
             %adaboost classification with alfa (weaklearner weights)
             %if negative: class = -1
             %else if positive: class = 1
             %else if zero: only the first cluster SVM prediction with maximum weight will be taken into account
         class = 0;
         for l=1:size(alfa,1);
            class = class + alfa(l) * trans_boosted_results (k,l).output(s);
         end;
         if (class == 0);
            [max_value max_idx] = max(alfa(:));
            class = trans_boosted_results (k,max_idx(1)).output(s);
         end;

         subj_predictions(s) = sign(class);

         %counting and compare with target label

         if (trans_boosted_results(k,:).target(s) == -1); %positive
            if   (subj_predictions(s) == -1); TP = TP + 1;
            else                              FN = FN + 1;
            end;
         else %negative
            if   (subj_predictions(s) ==  1); TN = TN + 1;
            else                              FP = FP + 1;
            end;
         end;

         total_tests = total_tests + 1;
         
         Partial_Accuracy    (k) = (TP + TN)/total_tests

         if (TN+FP ~= 0) Partial_Specificity (k) = TN / (TN+FP)
         else            Partial_Specificity (k) = 0;
         end;

         if (TP+FN ~= 0) Partial_Sensitivity (k) = TP / (TP+FN)
         else            Partial_Sensitivity (k) = 0;
         end;
      end;

      clear weaklearners subj_predictions subj_predictions_training trans_predict_label predict_label
      
k
   end %end k_folding


   Accuracy    = sum(Partial_Accuracy   (:))/ k_de_kfold
   Specificity = sum(Partial_Specificity(:))/ k_de_kfold
   Sensitivity = sum(Partial_Sensitivity(:))/ k_de_kfold



