%% reference: Xuchun Li, Lei Wang, Eric Sung - "A Study of AdaBoost with SVM Based Weak Learners" - IJCNN 2005
%% Para SVM la sigma del articulo es el learning rate (learn_rate_min = 0.001; learn_rate_max = 0.8; learn_rate_step = 0.05)
%% Para LVQ la sigma del articulo es el numero de hidden_neurons (hidden_neurons_min = 3; hidden_neurons_max = 20; hidden_neurons_step=1)

function [Accuracy Specificity Sensitivity results] = Diverse_AdaBoost_SVM (spm_vol, control_data, patient_data)
   
   [L, num_clusters] = bwlabeln (spm_vol);

   C = control_data;
   P = patient_data;

   k_de_kfold = 10;
   p = cvpartition(size(P,1),'kfold',k_de_kfold);
   c = cvpartition(size(C,1),'kfold',k_de_kfold);
   
   control_label =  1;
   patient_label = -1;

   sigma_init    = 50.0;
   sigma_min     =  0.1;
   sigma_step    =  0.1;
   min_diversity =  0.5;
   normalization_constant = 100;  
 
   for k=1:k_de_kfold;

      sigma = sigma_init;      
 
      %classify and evaluate adaboost
      total_tests = 0;
      TP = 0;
      TN = 0;
      FP = 0;
      FN = 0;
       
      n  = 1;
   
      %compute folds
      %training set used to compute the weights for each clusterSVM (our weaklearners)
      C_training = C(training (c,k) == 1,:,:);
      P_training = P(training (p,k) == 1,:,:);
   
      %test set used to validate the adaboost
      C_test     = C(training (c,k) == 0,:,:);
      P_test     = P(training (p,k) == 0,:,:);

      training_patients_no = size(P_training,1);
      training_controls_no = size(C_training,1);
      training_subjects_no = training_patients_no + training_controls_no;
      sample_weights       = ones(1,training_subjects_no)'/training_subjects_no; %sample_weights [#training_subjects x n]

      %initialize training set
      [training_data training_label feat_no] = alignAndLabelDataForNN (C_training, P_training, control_label, patient_label);
     
      %weight training
      while (sigma > sigma_min);
         gamma = 1/(2*sigma^2);         
 
         weaklearner = svmtrain (training_label, training_data, sprintf ('-s 0 -t 2 -c 1 -g %f', gamma) );

         %classification test with training data
         [predict_label, accuracy, prob_estimates] = svmpredict (training_label, training_data, weaklearner);

         %computing weaklearner's training error
         training_error = 0;
         for s=1:size(training_data,1);
             %adaboost compute training errors ¿ht(xi)=yi?
             if (predict_label(s) ~= training_label(s));
                training_error = training_error + sample_weights(s,n);
             end;
	 end;

         %(n-1) indicates the number of weaklearners selected for the meta-classifier
         if (n > 1);
            aux = 0;
            for s=1:size(training_data,1); 
                %%Calculo Diversity ¿ht(xi)= f(xi)?
                if (predict_label(s) ~= subj_predictions_training(s));
                    diversity = diversity + 1;
                end
            end;
            diversity = diversity./size(training_data,1);
         else %% n == 0; la primera vez 'subj_predictions_training' no existe
            diversity = min_diversity;
         end
        
%         training_error
%         diversity
         
         if ( (training_error > 0.50) || ( diversity < min_diversity ) )
            sigma = sigma - sigma_step;
         else
            %save the weaklearner, save the world!
            weaklearners         (n)   = weaklearner;

            prediction_labels    (n,:) = predict_label;
            probability_estimates(n,:) = prob_estimates;
            accuracies           (n,:) = accuracy;
            sigmas               (n,:) = sigma;

            training_errors      (n,:) = training_error;
            diversities          (n,:) = diversity;

            %adaboost set classifier weights
            alpha(n,:) = 1;
            if (training_errors(n) ~= 0);
               alpha (n) = 0.5 * log((1-training_errors(n)) / training_errors(n));
            end;

            sample_weights (:,n+1) = sample_weights(:,n)   .* exp( -alpha(n) .* training_label .* predict_label);
            sample_weights (:,n+1) = sample_weights(:,n+1) ./ normalization_constant;
            
            %%inicializamos/actualizamos 'subj_predictions_training', es decir, f(x)
            subj_predictions_training = zeros (1, training_subjects_no);
            for x=1:size(alpha,1);
                subj_predictions_training = subj_predictions_training + (alpha(x) .* predict_label');
                subj_predictions_training = sign(subj_predictions_training);
            end;

            n = n + 1;
         end;
      end;


      %classify test set with the meta-classifier
      [test_data, test_label] = alignAndLabelData (C_test, P_test, control_label, patient_label);

      for x=1:size(alpha,1);

         [boosted_results(k,x).output, boosted_results(k,x).accuracy, boosted_results(k,x).prob_estimates] = svmpredict (test_label, test_data, weaklearners(x));
          boosted_results(k,x).target = test_label;

         [weakres(k,x).TP weakres(k,x).TN weakres(k,x).FP weakres(k,x).FN weakres(k,x).Accuracy weakres(k,x).Specificity weakres(k,x).Sensitivity] = computeErrors (boosted_results(k,x).target, boosted_results(k,x).output);

      end;

      %saving k-fold results
      results(k).prediction_labels     = prediction_labels;
      results(k).accuracies            = accuracies;
      results(k).probability_estimates = probability_estimates;
      results(k).training_errors       = training_errors;
      results(k).diversities           = diversities;
      results(k).sigmas                = sigmas;

      results(k).adaboost_results      = boosted_results;
      results(k).weaklearners_results  = weakres;
    
      %check meta-classifier predictions   
      for s=1:size(test_data,1);
         
         %calculamos f(x)
             %adaboost classification with alfa (weaklearner weights)
             %if negative: class = -1
             %else if positive: class = 1
             %else if zero: only the first cluster SVM prediction with maximum weight will be taken into account
         class = 0;
         for l=1:size(alpha,1);
            class = class + alpha(l) * boosted_results (k,l).output(s);
         end;

         if (class == 0);
            [max_value max_idx] = max(alfa(:));
            class = trans_boosted_results (k,max_idx(1)).output(s);
         end;

         subj_predictions(s) = sign(class);

         %counting and compare with target label

         if (test_label(s) == -1); %positive
            if   (subj_predictions(s) == -1); TP = TP + 1;
            else                              FN = FN + 1;
            end;
         else %negative
            if   (subj_predictions(s) ==  1); TN = TN + 1;
            else                              FP = FP + 1;
            end;
         end;

         total_tests = total_tests + 1;
         
         Partial_Accuracy    (k) = (TP + TN)/total_tests

         if (TN+FP ~= 0) Partial_Specificity (k) = TN / (TN+FP)
         else            Partial_Specificity (k) = 0;
         end;

         if (TP+FN ~= 0) Partial_Sensitivity (k) = TP / (TP+FN)
         else            Partial_Sensitivity (k) = 0;
         end;
      end;

      clear weaklearners prediction_labels accuracies probability_estimates sigmas training_errors diversities alpha
    
k
   end %end k_folding


   Accuracy    = sum(Partial_Accuracy   (:))/ k_de_kfold
   Specificity = sum(Partial_Specificity(:))/ k_de_kfold
   Sensitivity = sum(Partial_Sensitivity(:))/ k_de_kfold



