%control_data (subject, cluster, features)
%patient_data (subject, cluster, features)
%libsvm_parameters    : (string)
%alpha                : vector with one weight value for each clusterSVM
function [alpha] = Train_ClusterSVM_AdaBoost (control_data, patient_data, libsvm_parameters)

   v                 = 10;
   num_clusters      = size(control_data,2);

   control_subjs_no  = size(control_data,1);
   control_partition = cvpartition(control_subjs_no,'kfold', v);

   patient_subjs_no  = size(patient_data,1);
   patient_partition = cvpartition(patient_subjs_no,'kfold', v);

   subjects_no       = patient_subjs_no + control_subjs_no;
   sample_weight     = 1/subjects_no;

   for k=1:v;

      %compute folds
      %training set used to compute the weights for each clusterSVM (our weaklearners)
      C_training = C(training (control_partition,k) == 1,:,:);
      P_training = P(training (patient_partition,k) == 1,:,:);

      %test set used to validate the adaboost
      C_test     = C(training (control_partition,k) == 0,:,:);
      P_test     = P(training (patient_partition,k) == 0,:,:);

      for i=1:num_clusters;
         %training
         [weight_training_data(i).data, weight_training_data(i).label] = alignAndLabelData (C_training(:,i,:), P_training(:,i,:));
         models(i) = svmtrain (weight_training_data(i).labels, weight_training_data(i).data, libsvm_parameters);

         %write training data to text
         path=strcat('cluster',num2str(i),'_trainingdata_', num2str(k),'.txt');
         writeDataToLIBSVMFormat (path, training_data, training_label);

         %classification
         [test_data(i).data, test_data(i).label] = alignAndLabelData (C_test(:,i ,:), P_test(:,i ,:));
         [results(k,i).predict_label, results(k,i).accuracy, results(k,i).prob_estimates] = svmpredict (test_data(i).label, test_data(i).data, models(i));

         %write test data to text
         path=strcat('cluster',num2str(i),'_testdata_', num2str(k),'.txt');
         writeDataToLIBSVMFormat (path, test_data, test_label);

         %adaboost compute training errors
         training_errors(k,i) = 0;
         for s=1:size(test_data(i).data,1);
            if (results (k,i).predict_label(s) ~= test_data(i).label(s));
               training_errors(k,i) = training_errors(k,i) + sample_weight;
            end;
         end;

         %adaboost set classifier weights
         alpha (k,i) = log((1-training_errors(k,i)) / training_errors(k,i)) / 2;

      end;
   end;

